package errors

import (
	"fmt"
	"github.com/gogf/gf/errors/gcode"
	"github.com/gogf/gf/errors/gerror"
	"strings"
	"sync"
)

var (
	codeNil      			 	= BizCode{0, "", ""}
	CodeSuccess                 = BizCode{1, "操作成功", ""}                           // It is success.
	CodeInternalError     	 	= BizCode{50, "Internal Error", ""}
	CodeUnknown     	 		= BizCode{500, "An internal server error occurred", ""}
)

type BizCode struct {
	// C refers to the integer code of the ErrCode.
	C    int

	// M (user) facing error text.
	M string

	// Ref specify the reference document.
	Ref 	interface{}
}

func init() {
	codes[CodeUnknown.Code()] = CodeUnknown
}

func (c BizCode) Code() int {
	return c.C
}

func (c BizCode) Message() string {
	return c.M
}


func (c BizCode) Detail() interface{} {
	return c.Ref
}

var codes = map[int]gcode.Code{}
var codeMux = &sync.Mutex{}

func MustRegister(code int, message string) {
	if code  < 1000  {
		panic("Codes less than 1000 are reserved as internal codes")
	}
	codeMux.Lock()
	defer codeMux.Unlock()

	if _, ok := codes[code]; ok {
		panic(fmt.Sprintf("code: %d already exist", code))
	}

	codes[code] = BizCode{
		C: code,
		M: message,
	}
}

func ParseCoder(err error) gcode.Code {
	code := gerror.Code(err)
	if code == gcode.CodeNil && err != nil {
		code = CodeInternalError
	}

	if v, ok := code.(BizCode); ok {
		if coder, ok := codes[v.C]; ok {
			return BizCode{
				C:    	coder.Code(),
				M: 		strings.Replace(coder.Message(),"%%0", v.M, -1),
				Ref : 	coder.Detail(),
			}
		}
	}

	return CodeUnknown
}

func New( code int, message string, reference string) error {
	return gerror.NewCode(BizCode{
		C:    code,
		M: message,
		Ref : reference,
	})
}