package response

import (
	"github.com/gogf/gf/net/ghttp"
	"github.com/gogf/gf/util/gconv"
	"github.com/gogf/gf/util/grand"
	"gitlab.jxhh.com/zhaoxueru/common-base.git/pkg/errors"
)

// 数据返回通用JSON数据结构
type JsonResponse struct {
	Id 		string 		`json:"id"`		//请求id
	Code    int         `json:"code"`    // 错误码((1:成功, >1:错误码))
	Message string      `json:"message"` // 提示信息
	Data    interface{} `json:"data"`    // 返回数据(业务接口定义具体数据结构)
	Redirect string      `json:"redirect,omitempty"` // 引导客户端跳转到指定路由
}

// 标准返回结果数据结构封装。
func Json(r *ghttp.Request, code int, message string, data ...interface{}) {
	responseData := interface{}(nil)
	if len(data) > 0 {
		responseData = data[0]
	}
	jsonResponse := new(JsonResponse)

	jsonResponse.Code = code
	jsonResponse.Message = message
	jsonResponse.Data = responseData
	jsonResponse.Id = grand.S(20)
	if !r.GetCtxVar("RequestId").IsNil(){
		jsonResponse.Id = gconv.String(r.GetCtxVar("RequestId"))
	}
	r.Response.WriteJson(jsonResponse)

}



// 返回JSON数据并退出当前HTTP执行函数。
func JsonExit(r *ghttp.Request, err error, data interface{}) {
	if err != nil {
		coder := errors.ParseCoder(err)
		Json(r, coder.Code(), coder.Message(), data)
	}else {
		Json(r, errors.CodeSuccess.Code(), errors.CodeSuccess.Message(), data)
	}
	r.Exit()
}


